@@ -1,19 +1,19 @@
-# Modified: 12 May 2016 SDH
 
-interp <-
-function(times, y, time.out, method = 'linear', extrap = FALSE) {
+interp <- function(times, y, time.out, method = 'linear', extrap = FALSE) {
 
   # Check arguments
   checkArgClassValue(times, c('numeric', 'integer', 'POSIXct', 'POSIXlt', 'POSIXt', 'POSIX', 'difftime'))
   checkArgClassValue(y, c('numeric', 'integer'))
   checkArgClassValue(time.out, c('numeric', 'integer', 'POSIXct', 'POSIXlt', 'POSIXt', 'POSIX'))
-  checkArgClassValue(method, c('character'), expected.values = c('linear', 'natural', 'hyman', 'fmm'))
+  checkArgClassValue(method, c('character'), expected.values = c('linear', 'natural', 'hyman', 'fmm', 'f1'))
   checkArgClassValue(extrap, 'logical')
 
   #if(extrap & (max(time.out)>max(times) | min(time.out)<min(times))) warning('Some results are extrapolated. Set extrap = FALSE to prevent this.')
 
   if(method=='linear') {
-    y.out <- approx(times, y, xout = time.out, rule = ifelse(extrap, 2, 1))$y
+    y.out <- approx(times, y, xout = time.out, method = 'linear', rule = ifelse(extrap, 2, 1))$y
+  } else if(tolower(method) == 'f1') { 
+    y.out <- approx(times, y, xout = time.out, method = 'constant', rule = 2, f = 1)$y
   } else { 
     # Or for natural, hyman, or fmm methods (others will return an error)
     y.out <- spline(times, y, xout = time.out, method = method)$y
